Linux  R2.6.5-7.283-default FORTRAN90/SX         Rev.360        Wed Oct 22 11:35:46 2008
FILE NAME: i.lpj_crown.f90
PROGRAM NAME: lpj_crown
DIAGNOSTIC LIST

  LINE  LEVEL( NO.): DIAGNOSTIC MESSAGE

    83  opt  (1593): Loop nest collapsed into one loop.
    83  vec  (   4): Vectorized array expression.
    85  vec  (   4): Vectorized array expression.
    89  vec  (   3): Unvectorized loop.
    98  vec  (   4): Vectorized array expression.
   106  vec  (   4): Vectorized array expression.
   112  vec  (   4): Vectorized array expression.
   122  vec  (   4): Vectorized array expression.
   132  vec  (   4): Vectorized array expression.
Linux  R2.6.5-7.283-default FORTRAN90/SX         Rev.360        Wed Oct 22 11:35:46 2008
FILE NAME: i.lpj_crown.f90
PROGRAM NAME: lpj_crown
FORMAT LIST

  LINE    LOOP     FORTRAN STATEMENT

     1:            ! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_stomate/lpj_crown.f90,v 1.11 2007/05/28 14:49:02 ssipsl Exp $
     2:            ! IPSL (2006)
     3:            !  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
     4:            !-
     5:            MODULE lpj_crown
     6:            !---------------------------------------------------------------------
     7:            !- calculate individual crown area from stem mass.
     8:            !---------------------------------------------------------------------
     9:              USE ioipsl
    10:              USE stomate_constants
    11:              USE constantes_veg
    12:            !-
    13:              IMPLICIT NONE
    14:            !-
    15:            ! private & public routines
    16:            !-
    17:              PRIVATE
    18:              PUBLIC crown
    19:            !-
    20:            CONTAINS
    21:            !-
    22:            !===
    23:            !-
    24:            SUBROUTINE crown &
    25:             &  (npts, PFTpresent, ind, biomass, veget_max, cn_ind, height)
    26:            !---------------------------------------------------------------------
    27:            ! 0 declarations
    28:            !-
    29:            ! 0.1 input
    30:            !-
    31:            ! Domain size
    32:              INTEGER(i_std),INTENT(in) :: npts
    33:            ! Is pft there
    34:              LOGICAL,DIMENSION(npts,npft),INTENT(in) :: PFTpresent
    35:            ! density of individuals (1/(m**2 of nat/agri ground))
    36:              REAL(r_std),DIMENSION(npts,npft),INTENT(in) :: ind
    37:            ! biomass (gC/(m**2 of nat/agri ground))
    38:              REAL(r_std),DIMENSION(npts,npft,nparts),INTENT(in) :: biomass
    39:            !-
    40:            ! 0.2 modified fields
    41:            !-
    42:            ! "maximal" coverage fraction of a PFT (LAI -> infinity)
    43:            ! on nat/agri ground
    44:            !-
    45:              REAL(r_std),DIMENSION(npts,npft),INTENT(inout) :: veget_max
    46:            !-
    47:            ! 0.3 output
    48:            !-
    49:            ! crown area (m**2) per ind.
    50:            !-
    51:              REAL(r_std),DIMENSION(npts,npft),INTENT(out) :: cn_ind
    52:            !-
    53:            ! height of vegetation (m)
    54:            !-
    55:              REAL(r_std),DIMENSION(npts,npft),INTENT(inout) :: height
    56:            !-
    57:            ! 0.4 local
    58:            !-
    59:            ! wood mass of an individual
    60:            !-
    61:              REAL(r_std),DIMENSION(npts) :: woodmass
    62:            !-
    63:            ! index
    64:            !-
    65:              INTEGER(i_std) :: j
    66:            !-
    67:            ! stem diameter
    68:            !-
    69:              REAL(r_std),DIMENSION(npts) :: dia
    70:              REAL(r_std),DIMENSION(npft) :: height_presc_12
    71:            !---------------------------------------------------------------------
    72:            !-
    73:            ! 1 initializations
    74:            !-
    75:            ! 1.1 check if DGVM activated
    76:            !-
    77:              IF (.NOT.control%ok_dgvm) THEN
    78:                STOP 'crown: not to be called with static vegetation.'
    79:              ENDIF
    80:            !-
    81:            ! 1.2 initialize output to zero
    82:            !-
    83: W+=====      cn_ind(:,:) = 0.0
    84:            ! convert prescribed height from sechiba (nvm) to stomate (npft) 
    85: V======      height_presc_12(1:npft) = height_presc(2:nvm)
    86:            !-
    87:            ! 2 calculate (or prescribe) crown area
    88:            !-
    89: +------>     DO j = 1,npft
    90: |              IF (tree(j)) THEN
    91: |          !-----
    92: |          !---- 2.1 trees
    93: |          !-----
    94: |                IF (natural(j)) THEN
    95: |          !------ 2.1.1 natural
    96: |                  WHERE (PFTpresent(:,j))
    97: |          !-------- 2.1.1.1 calculate individual wood mass
    98: |V----->             woodmass(:) = &
    99: ||          &         (biomass(:,j,isapabove)+biomass(:,j,isapbelow) &
   100: ||          &         +biomass(:,j,iheartabove)+biomass(:,j,iheartbelow))/ind(:,j)
   101: ||         !-------- 2.1.1.2 stem diameter (pipe model)
   102: ||                   dia(:) = (woodmass(:)/(pipe_density*pi/4.*pipe_tune2)) &
   103: ||          &                **(1./(2.+pipe_tune3))
   104: ||         !-------- 2.1.1.3 height
   105: |V-----              height(:,j) = pipe_tune2*(dia(:)**pipe_tune3)
   106: |V----->             WHERE (height(:,j) > height_presc_12(j))
   107: ||                     dia(:) = (height_presc_12(j)/pipe_tune2)**(1./pipe_tune3)
   108: |V-----                height(:,j) = height_presc_12(j)
   109: |                    ENDWHERE
   110: |          !-------- 2.1.1.4 crown area: for large truncs, crown area cannot
   111: |          !--------         exceed a certain value, prescribed through maxdia.
   112: |V=====              cn_ind(:,j) = pipe_tune1*MIN(dia(:),maxdia(j))**1.6
   113: |                  ENDWHERE
   114: |                ELSE
   115: |          !------ 2.1.2 tree is agricultural - stop
   116: |                  STOP 'crown: cannot treat agricultural trees.'
   117: |                ENDIF
   118: |              ELSE
   119: |          !-----
   120: |          !---- 2.2 grasses
   121: |          !-----
   122: |V----->         WHERE (PFTpresent(:,j))
   123: ||         !------ 2.2.1 an "individual" is 1 m**2 of grass
   124: |V-----            cn_ind(:,j) = 1.
   125: |                ENDWHERE
   126: |              ENDIF
   127: |          !---
   128: |          !-- 2.3 recalculate vegetation cover if natural
   129: |          !       ind and cn_ind are 0 if not present
   130: |          !---
   131: |              IF (natural(j)) THEN
   132: |V=====          veget_max(:,j) = ind(:,j) * cn_ind(:,j)
   133: |              ENDIF
   134: +------      ENDDO
   135:            !-------------------
   136:            END SUBROUTINE crown
   137:            !-
   138:            !===
   139:            !-
   140:            END MODULE lpj_crown
