!
! update/completion by P. Smith
!

! Stomate: deforestation
!
! authors: M. Boisserie, P. Friedlingstein
!
! 
!
! version 0.0: May 2003
!
! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_stomate/stomate_deforestation.f90,v 1.5 2007/06/13 07:55:43 ssipsl Exp $
! IPSL (2006)
!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
MODULE stomate_deforestation


  ! modules used:

  USE ioipsl
  USE stomate_constants
  USE stomate_data

  IMPLICIT NONE


  PRIVATE
  PUBLIC deforestation

CONTAINS

  SUBROUTINE deforestation(npts, dt_days, space_nat, space_nat_new,  veget_max, veget_max_new,&
       biomass, ind, age, PFTpresent, senescence, when_growthinit, everywhere, veget,&        
       co2_to_bm, bm_to_litter, bm_sapl, tree, cn_ind,flux10,flux100, &
       prod10,prod100,prod10_total,prod100_total,&
       convflux,cflux_prod_total,cflux_prod10,cflux_prod100, leaf_frac,&
       npp_longterm, lm_lastyearmax)

    ! 0 declarations

    ! 0.1 input

    ! Domain size
    INTEGER, INTENT(in)                                            :: npts

    ! Time step (days)
    REAL(r_std), INTENT(in)                                         :: dt_days 

    ! new "maximal" coverage fraction of a PFT (LAI -> infinity) on nat/agri ground
    REAL(r_std), DIMENSION(npts,npft), INTENT(in)                   :: veget_max_new 

    ! biomass of sapling (gC)
    REAL(r_std) , DIMENSION (npft, nparts), INTENT(in)              :: bm_sapl

    ! is pft a tree
    LOGICAL, DIMENSION(npft), INTENT(in)                           :: tree


    ! 0.2 modified fields

    ! total natural space (fraction of total space)
    REAL(r_std), DIMENSION(npts), INTENT(inout)                         :: space_nat

    ! total natural space (fraction of total space)
    REAL(r_std), DIMENSION(npts), INTENT(in)                         :: space_nat_new

    ! fractional coverage on natural/agricultural ground, taking into
    !   account LAI (=grid-scale fpc)
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                    :: veget

    ! "maximal" coverage fraction of a PFT (LAI -> infinity) on nat/agri ground
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                    :: veget_max

    ! biomass (gC/(m**2 of nat/agri ground))
    REAL(r_std), DIMENSION(npts,npft,nparts), INTENT(inout)             :: biomass

    ! density of individuals 1/m**2
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                    :: ind

    ! mean age (years)
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                    :: age

    ! is the plant senescent? (only for deciduous trees - carbohydrate reserve)
    !         set to .FALSE. if PFT is introduced or killed
    LOGICAL, DIMENSION(npts,npft), INTENT(inout)                       :: senescence

    ! PFT exists
    LOGICAL, DIMENSION(npts,npft), INTENT(inout)                       :: PFTpresent

    ! is the PFT everywhere in the grid box or very localized (after its introduction)
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                    :: everywhere

    ! how many days ago was the beginning of the growing season
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                    :: when_growthinit

    ! biomass uptaken (gC/(m**2 of total ground)/day)
    REAL(r_std), DIMENSION(npts), INTENT(inout)                       :: co2_to_bm

    ! conversion of biomass to litter (gC/(m**2 of nat/agri ground)) / day
    REAL(r_std), DIMENSION(npts,npft,nparts), INTENT(inout)           :: bm_to_litter

    ! crown area (m**2) per ind.
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                   :: cn_ind

    ! products remaining in the 10/100 year-turnover pool after the annual release for each compartment
    ! (10 or 100 + 1 : input from year of deforestation)
    REAL(r_std), DIMENSION(npts,0:10), INTENT(inout)                     :: prod10
    REAL(r_std), DIMENSION(npts,0:100), INTENT(inout)                    :: prod100

    ! annual release from the 10/100 year-turnover pool compartments
    REAL(r_std), DIMENSION(npts,10), INTENT(inout)                     :: flux10
    REAL(r_std), DIMENSION(npts,100), INTENT(inout)                    :: flux100 

    ! fraction of leaves in leaf age class
    REAL(r_std), DIMENSION(npts,npft,nleafages), INTENT(inout)         :: leaf_frac

    ! last year's maximum leaf mass, for each PFT (gC/(m**2 of nat/agri ground))
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                   :: lm_lastyearmax

    ! "long term" net primary productivity (gC/(m**2 of nat/agri ground)/year)
    REAL(r_std), DIMENSION(npts,npft), INTENT(inout)                    :: npp_longterm


    ! 0.3 output

    ! release during first year following deforestation
    REAL(r_std), DIMENSION(npts), INTENT(out)                          :: convflux

    ! total annual release from the 10/100 year-turnover pool
    REAL(r_std), DIMENSION(npts), INTENT(out)                          :: cflux_prod10, cflux_prod100

    ! total products remaining in the pool after the annual release 
    REAL(r_std), DIMENSION(npts), INTENT(out)                          :: prod10_total, prod100_total 

    ! total flux from conflux and the 10/100 year-turnover pool 
    REAL(r_std), DIMENSION(npts), INTENT(out)                          :: cflux_prod_total


    ! 0.4 local

    ! indices
    INTEGER(i_std)                                                     :: i, j, k, l, m

    ! biomass increase (gC/(m**2 of nat/agri ground))
    REAL(r_std),DIMENSION(npts)                                       :: bm_new

    ! change in number of individuals /m2 per year
    REAL(r_std),DIMENSION(npts,npft)                                  :: d_ind 

    REAL(r_std)                                                       :: x,above
    REAL(r_std),DIMENSION(npft)                                       :: y,z

    REAL(r_std), DIMENSION(npts,npft)                                 :: vegetspace
    REAL(r_std), DIMENSION(npts,npft)                                 :: vegetspace_new    

    ! =========================================================================

    ! yearly initialisation
    prod10(:,0)           = zero
    prod100(:,0)          = zero   
    above                 = zero
    convflux(:)           = zero
    cflux_prod10(:)       = zero
    cflux_prod100(:)      = zero
    prod10_total(:)       = zero
    prod100_total(:)      = zero
    cflux_prod_total(:)   = zero
    bm_new(:)             = zero

    DO i = 1, npts  

       DO j = 1, npft

          IF( natural(j) ) THEN

             vegetspace(i,j) = veget_max(i,j) * space_nat(i)
             vegetspace_new(i,j) = veget_max_new(i,j) * space_nat_new(i)

          ELSE

             vegetspace(i,j) = veget_max(i,j) *(1. - space_nat(i))
             vegetspace_new(i,j) = veget_max_new(i,j) *(1. - space_nat_new(i))

          ENDIF

          ! in case of establishment of a new PFT or extension of its coverage in a gridcell
          IF ( vegetspace_new(i,j) .GT. vegetspace(i,j)  ) THEN

             IF ( tree(j) ) THEN

                IF ( abs(cn_ind(i,j)) > 1e-16) THEN
                d_ind(i,j) =  (vegetspace_new(i,j) - vegetspace(i,j)) / cn_ind (i,j)
                ELSE
                   write(numout,*) "deforestation : cn_ind est nul pour i,j = ",i,j
                   d_ind(i,j) = zero
                ENDIF

             ELSE 

                d_ind(i,j) =  vegetspace_new(i,j) - vegetspace(i,j)

             ENDIF

             DO k = 1, nparts

                bm_new(i) = d_ind(i,j) * bm_sapl(j,k)
                biomass(i,j,k) = biomass(i,j,k) + bm_new(i)
  
                IF( natural(j) ) THEN                  

                   co2_to_bm(i) = co2_to_bm(i) + bm_new(i) * space_nat_new(i) / one_year * dt_days

                ELSE

                   co2_to_bm(i) = co2_to_bm(i) + bm_new(i) * (1.-space_nat_new(i)) / one_year * dt_days

                ENDIF

             ENDDO

             PFTpresent(i,j) = .TRUE.
             everywhere(i,j) = 1.
             senescence(i,j) = .FALSE.
             age(i,j) = 0.
             ind(i,j) = veget_max_new (i,j)
             when_growthinit(:,j) = large_value
             leaf_frac(:,j,1) = 1.0
             npp_longterm(:,j) = 10.
             lm_lastyearmax(:,j) = bm_sapl(j,ileaf) * ind(:,j)

          ENDIF   ! End if PFT extension

          ! in case of total or fractional disparition or a PFT
          IF ( vegetspace_new(i,j) .LT. vegetspace(i,j))     THEN

             x    = 40./67.
             y(1) = 27./67. 
             y(2) = 27./67. 
             y(9) = 27./67.      
             y(3) = 20./67.
             y(4) = 20./67.
             y(5) = 20./67.  
             y(6) = 20./67.
             y(7) = 20./67.
             y(8) = 20./67. 
             y(10)= 20./67. 
             y(11)= 20./67. 
             z(3) = 7./67. 
             z(4) = 7./67. 
             z(5) = 7./67.
             z(6) = 7./67.
             z(7) = 7./67.
             z(8) = 7./67.
             z(11)= 7./67.
             z(12)= 7./67.
             z(1) = 0.
             z(2) = 0.
             z(9) = 0. 

             IF ( vegetspace_new(i,j) .EQ. 0. ) THEN

                IF (.NOT. tree(j) ) THEN 

                   ind(i,j) = 0.0
                   bm_to_litter(i,j,:) = bm_to_litter(i,j,:) + biomass(i,j,:) 
                   biomass(i,j,:) = 0.0             

                ELSE 

                   ind(i,j) = 0.0
                   cn_ind(i,j) = 0.0
                   above =  biomass(i,j,1) +  biomass(i,j,2) + &
                        biomass(i,j,4)
                   convflux(i)  = convflux(i)  +  x    * above
                   prod10(i,0)  = prod10(i,0)  +  y(j) * above
                   prod100(i,0) = prod100(i,0) +  z(j) * above
                   bm_to_litter(i,j,3) = bm_to_litter(i,j,3) + biomass(i,j,3)
                   bm_to_litter(i,j,5) = bm_to_litter(i,j,5) + biomass(i,j,5) 
                   bm_to_litter(i,j,6) = bm_to_litter(i,j,6) + biomass(i,j,6)
                   bm_to_litter(i,j,7) = bm_to_litter(i,j,7) + biomass(i,j,7)
                   bm_to_litter(i,j,8) = bm_to_litter(i,j,8) + biomass(i,j,8)   
                   biomass(i,j,:) = 0.0

                ENDIF

                PFTpresent(i,j) = .FALSE.
                senescence(i,j) = .FALSE.
                age(i,j) = 0.0
                when_growthinit(i,j) = undef
                everywhere(i,j) = 0.0
                veget(i,j) = 0.0

             ELSE

                d_ind(i,j) =  vegetspace(i,j) - vegetspace_new(i,j)

                IF  (.NOT. tree (j))    THEN

                   bm_to_litter(i,j,:) = bm_to_litter(i,j,:) + &
                        biomass(i,j,:) * d_ind(i,j) / veget_max(i,j)
                   biomass(i,j,:) = biomass(i,j,:) * (1. - d_ind(i,j)/veget_max(i,j))
                   ind(i,j) = ind(i,j) * (1. - d_ind(i,j))

                ELSE

                   above =(biomass(i,j,1) + biomass(i,j,2) + biomass(i,j,4)) * d_ind(i,j)/veget_max(i,j)
                   convflux(i)  = convflux(i)  + x    * above
                   prod10(i,0)  = prod10(i,0)  + y(j) * above
                   prod100(i,0) = prod100(i,0) + z(j) * above
                   bm_to_litter(i,j,3) = bm_to_litter(i,j,3) + &
                        biomass(i,j,3)* d_ind(i,j)/vegetspace(i,j)
                   bm_to_litter(i,j,5) = bm_to_litter(i,j,5) + &
                        biomass(i,j,5)* d_ind(i,j)/vegetspace(i,j)
                   bm_to_litter(i,j,6) = bm_to_litter(i,j,6) + &
                        biomass(i,j,6)* d_ind(i,j)/vegetspace(i,j)
                   bm_to_litter(i,j,7) = bm_to_litter(i,j,7) +  &
                        biomass(i,j,7)* d_ind(i,j)/vegetspace(i,j)
                   bm_to_litter(i,j,8) = bm_to_litter(i,j,8) + &
                        biomass(i,j,8)* d_ind(i,j) /vegetspace(i,j)  
                   biomass(i,j,:) = biomass(i,j,:) * ( 1. - d_ind(i,j))/vegetspace(i,j) 

                ENDIF

             ENDIF   ! End if PFT total disparition

          ENDIF   ! End if PFT's coverage reduction

       ENDDO   ! End loop on PFTs

       ! each year, update 10 year-turnover pool content following flux emission
       ! (linear decay (10%) of the initial carbon input)
       DO   l = 0, 8
          m = 10 - l
          cflux_prod10(i) =  cflux_prod10(i) + flux10(i,m)
          prod10(i,m)     =  prod10(i,m-1)   - flux10(i,m-1)
          prod10_total(i) =  prod10_total(i) + prod10(i,m)
          flux10(i,m)     =  flux10(i,m-1)

          IF (prod10(i,m) .LT. 1.0) prod10(i,m) = 0.0

       ENDDO

       cflux_prod10(i) = cflux_prod10(i) + flux10(i,1) 
       flux10(i,1)     = 0.1 * prod10(i,0)
       prod10(i,1)     = prod10(i,0)
       prod10_total(i) = prod10_total(i)  + prod10(i,1)

       DO   l = 0, 98
          m = 100 - l
          cflux_prod100(i)  =  cflux_prod100(i) + flux100(i,m)
          prod100(i,m)      =  prod100(i,m-1)   - flux100(i,m-1)
          prod100_total(i)  =  prod100_total(i) + prod100(i,m)
          flux100(i,m)      =  flux100(i,m-1)

          IF (prod100(i,m).LT.1.0) prod100(i,m) = 0.0

       ENDDO

       cflux_prod100(i)  = cflux_prod100(i) + flux100(i,1) 
       flux100(i,1)      = 0.01 * prod100(i,0)
       prod100(i,1)      = prod100(i,0)
       prod100_total(i)  = prod100_total(i) + prod100(i,1)
       cflux_prod_total(i) = convflux(i) + cflux_prod10(i) + cflux_prod100(i)
       prod10(i,0)        = 0.0
       prod100(i,0)       = 0.0 

    ENDDO  ! End loop on npts

    space_nat(:) = space_nat_new(:)
    veget_max(:,:) = veget_max_new(:,:)

    ! convert flux from /year into /time step
    convflux        = convflux/one_year*dt_days
    cflux_prod10    = cflux_prod10/one_year*dt_days
    cflux_prod100   = cflux_prod100/one_year*dt_days

  END SUBROUTINE deforestation

END MODULE stomate_deforestation
